package tony
{
	import flash.ui.MouseCursor;
	import flash.ui.Mouse;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.utils.Input;
	
	/**
	 * ...
	 * @author MakotoNinja
	 */
	public class SimpleButton extends Entity
	{
		public static const DEFAULT:uint  = 0;
		public static const OVER:uint	  = 1;
		public static const DOWN:uint 	  = 2;
		public static const DISABLED:uint = 3;
		
		protected var _status:uint;
		protected var _callback:Function;
		protected var _gfk:Spritemap;
		
		/**
		 * 
		 * @param		  px: Number	- desired x position of the button
		 * @param		  py: Number	- desired y position of the button
		 * @param		 gfk: Spritemap - a spritemap that contains an image with exactly 4 states (default, over, down, disabled)
		 * @param	callback: Function  - a function to be run when the mouse is clicked
		 */
		public function SimpleButton(px:Number, py:Number, gfk:Spritemap, callback:Function = null)
		{
			super(px, py);
			
			graphic = _gfk = gfk;
			_gfk.add("states", [SimpleButton.DEFAULT, SimpleButton.OVER, SimpleButton.DOWN, SimpleButton.DISABLED]);
			setHitboxTo(graphic);
			
			_callback = callback;
			_status = SimpleButton.DEFAULT;
		}
		
		override public function update():void
		{
			if (!collidePoint(x, y, world.mouseX, world.mouseY) && (_status == OVER || _status == DOWN) ) onMouseLeave();
			if ( collidePoint(x, y, world.mouseX, world.mouseY) && _status == DEFAULT) onMouseEnter();
			if (_status == OVER && Input.mousePressed) onMousePress();
			if (_status == DOWN && Input.mouseReleased) onMouseRelease();
			
			super.update();
		}
		
		/**
		 * Runs when the mouse is over the button's hitbox
		 */
		public function onMouseEnter():void
		{
			_status = _gfk.frame = OVER;
			Mouse.cursor = MouseCursor.BUTTON;
		}
		/**
		 * Runs when the mouse leaves the button's hitbox
		 */
		public function onMouseLeave():void
		{
			_status = _gfk.frame = DEFAULT;
			Mouse.cursor = MouseCursor.AUTO;
		}
		/**
		 * Runs if the mouse button was pressed
		 */
		public function onMousePress():void
		{
			_status = _gfk.frame = DOWN;
		}
		/**
		 * Runs when the button is released
		 */
		public function onMouseRelease():void
		{
			try{
				if (_callback != null) _callback();
			} catch (e:Error) {
				if (e.errorID == 1063) _callback(this);
				else throw new Error(e);
			}
			if (_status == DOWN && collidePoint(x, y, world.mouseX, world.mouseY)) onMouseEnter();
		}
		
		/**
		 * enables the button
		 */
		public function enable():void
		{
			_status = _gfk.frame = DEFAULT;
		}
		
		/**
		 * disables the button
		 */
		public function disable():void
		{
			_status = _gfk.frame = DISABLED;
		}
		
		public function get status():uint { return _status; }
		public function set status(v:uint):void { _status = v; }
	}
}